"""
Dump Glottolog data as CLDF dataset
"""
import pathlib

from clldutils import jsonlib

from pyglottolog import cldf


def register(parser):
    parser.add_argument(
        'cldf_dir',
        type=pathlib.Path,
        help='path to clone of glottolog/glottolog-cldf')


def run(args):
    cldf.cldf(args.repos, args.cldf_dir, args.log)
    zenodo = jsonlib.load(args.repos.path('.zenodo.json'))
    zenodo['title'] = zenodo['title'].replace('/glottolog:', '/glottolog-cldf:') + ' as CLDF'
    zenodo['communities'].append(dict(identifier='cldf-datasets'))
    zenodo['keywords'].append('cldf:StructureDataset')
    jsonlib.dump(zenodo, args.cldf_dir.parent / '.zenodo.json', indent=4)



from html import entities as html_entities

from nameparser import HumanName
from markdown import markdown
from clldutils.jsonlib import dump


def to_html(text, url):
    c2n = html_entities.codepoint2name
    text = text.replace('&', '&amp;')
    text = ''.join('&{0};'.format(
        c2n[ord(c)]) if ord(c) > 128 and ord(c) in c2n else c for c in text)
    return markdown(text.replace(url, '[{0}]({0})'.format(url)))


def read_editors(repos):
    res = []
    in_editors, in_table = False, False
    for line in repos.path('CONTRIBUTORS.md').read_text(encoding='utf8').split('\n'):
        line = line.strip()
        if line.startswith('##'):
            if in_editors:
                in_editors = False
            elif line.endswith('Editors'):
                in_editors = True
            continue

        if line.startswith('---'):
            in_table = True
            continue

        if in_editors and in_table and line.strip():
            row = [m.strip() for m in line.split('|')]
            row[2] = [n.strip() for n in row[2].split('&')]
            res.append(row)
    return sorted(res, key=lambda t: tuple(map(int, t[0].split('.'))), reverse=True)


def editor_to_dict(n, editors):
    res = {'name': n}
    for e in editors.values():
        if e.name == n:
            assert e.current, '{} is not marked as current editor'.format(n)
            if e.affiliation:
                res['affiliation'] = e.affiliation
            if e.orcid:
                res['orcid'] = e.orcid
            break
    return res


def prepare_release(repos, version):
    for v, year, editors in read_editors(repos):
        if v == version:
            break
    else:  # pragma: no cover
        raise ValueError('Add version to CONTRIBUTORS.md first!')

    citation = "{0}. {1}. {2} {3}. {4}: {5}. (Available online at {6})".format(
        ' & '.join('{0.last}, {0.first}'.format(HumanName(e)) for e in editors),
        year,
        repos.publication.web.name,
        version,
        repos.publication.publisher.place,
        repos.publication.publisher.name,
        repos.publication.web.url,
    )
    dump(
        {
            "title": repos.publication.zenodo.title_format.format(version),
            "description": to_html(citation, repos.publication.web.url),
            "license": {"id": repos.publication.zenodo.license_id},
            "keywords": repos.publication.zenodo.keywords.split(),
            "communities": [
                {"identifier": cid} for cid in repos.publication.zenodo.communities.split()],
            "creators": [editor_to_dict(n, repos.editors) for n in editors],
            "access_right": "open",
            "upload_type": "dataset",
        },
        repos.path('.zenodo.json'),
        indent=4)
    return citation



import re
import functools
import collections

import requests
import attr

from csvw.dsv import reader
from clldutils.misc import nfilter
from clldutils.attrlib import valid_range

from .util import LinkProvider

BASE_URL = "http://endangeredlanguages.com"
CSV_URL = BASE_URL + "/userquery/download/"


def split(s, sep=';'):
    s = re.sub('"(?P<name>[^;]+);"', lambda m: '"{}";'.format(m.group('name').strip()), s)
    return nfilter(ss.strip() for ss in s.split(sep))


def parse_coords(s):
    cc = nfilter(ss.strip().replace(' ', '') for ss in re.split('[,;]', s))
    return [Coordinate(cc[i], cc[i + 1]) for i in range(0, len(cc), 2)]


@attr.s
class Coordinate(object):
    latitude = attr.ib(converter=lambda s: float(s.strip()), validator=valid_range(-90, 90))
    longitude = attr.ib(converter=lambda s: float(s.strip()), validator=valid_range(-180, 180))


@attr.s
class ElCatLanguage(object):
    id = attr.ib(converter=int)
    isos = attr.ib(converter=functools.partial(split, sep=','))
    name = attr.ib(converter=lambda s: s.strip())
    also_known_as = attr.ib(converter=split)
    status = attr.ib()
    speakers = attr.ib()
    classification = attr.ib()
    variants_and_dialects = attr.ib(converter=split)
    u = attr.ib()
    comment = attr.ib()
    countries = attr.ib(converter=split)
    continent = attr.ib()
    coordinates = attr.ib(converter=parse_coords)

    @property
    def names(self):
        return [self.name] + [n for n in self.also_known_as if n != self.name]

    @property
    def url(self):
        return BASE_URL + '/lang/{0.id}'.format(self)


def read():
    return [ElCatLanguage(*row) for row in reader(requests.get(CSV_URL).text.split('\n')) if row]


class ElCat(LinkProvider):
    def iterupdated(self, languoids):
        elcat_langs = collections.defaultdict(list)
        for line in read():
            for iso in line.isos:
                elcat_langs[iso].append(line)

        for lang in languoids:
            changed = False
            if lang.iso in elcat_langs:
                if lang.update_links(
                    'endangeredlanguages.com', [(l_.url, l_.name) for l_ in elcat_langs[lang.iso]]
                ):
                    changed = True
                if len(elcat_langs[lang.iso]) == 1:
                    # Only add alternative names, if only one ElCat language matches!
                    changed = lang.update_names(
                        elcat_langs[lang.iso][0].names, type_='elcat') or changed
            else:
                changed = any([lang.update_links('endangeredlanguages.com', []),
                               lang.update_names([], type_='elcat')])

            if changed:
                yield lang



"""
Write release info to .zenodo.json, CITATION.md and CONTRIBUTORS.md
"""
from pyglottolog.metadata import prepare_release


def register(parser):
    parser.add_argument('version', help="version number without leading 'v'")


def run(args):
    print(prepare_release(args.repos, args.version))
